//////////////////////////////////////////////////////////////////////////
// Piles pour ZnZ
//------------------------------------------------------------------------
// Cette unit, qui accompagne ZnZ 6.1, a une implmentation diffrente
// de l'unit ZnZ_Stacks des versions prcdentes.
// Les fichiers sauvegards avec la version prcdente peuvent tre
// chargs avec celle-ci.
// L'interface n'a subi qu'une seule modification: l'ajout de la mthode
// Sort qui permet de trier le contenu de la pile. Sort peut tre appele
// avec les fonctions Cmpn et CmpAbs de ZnZ.
//------------------------------------------------------------------------
// Copyright (c) 1997-98, Marcel Martin. Tous droits rservs.
//////////////////////////////////////////////////////////////////////////
unit ZnZ_Stacks;
{$A+,H+}

interface
{$IFDEF VER90}  //DELPHI 2
uses ZnZ_d2, Classes;
{$ELSE}
{$IFDEF VER100} //DELPHI 3
uses ZnZ_d3, Classes;
{$ENDIF}
{$ENDIF}

//===== Objet "Pile"
type
  TStackSortCompare = TListSortCompare;
  TStack = class
  private
    FX: TList;
    function GetCount: Integer;
  public
    constructor Create;
    destructor  Destroy; override;
    //---------------------------------------------
    procedure Flush;
    procedure LoadFromFile(const FileName: string);
    function  Pop(N: TNumber): Boolean;
    function  PopDown(N: TNumber): Boolean;
    procedure Push(N: TNumber);
    procedure SaveToFile(const FileName: string);
    procedure Sort(Compare: TStackSortCompare);
    //---------------------------------------------
    property Count: Integer read GetCount;
  end;

implementation
//=======================================================================
{$O+,Q-,R-}
{$IFDEF VER100} //DELPHI 3
  {$C-,G-}
{$ENDIF}
//=======================================================================
uses SysUtils;

/////////////////////////////////////////////////////////////////////////
// Type TStack
/////////////////////////////////////////////////////////////////////////

//----- Utiliss par LoadFromFile et SaveToFile
type
  THeaderStack = array[0..9] of Char;
const
  ZnZHEADERSTACK : THeaderStack = 'ZnZ Stack ';

//=======================================================================
//=======================================================================
constructor TStack.Create;
begin
  FX := TList.Create;
end;

//=======================================================================
//=======================================================================
destructor TStack.Destroy;
begin
  Flush;
  FX.Free;
end;

//=======================================================================
// Vide la pile
//=======================================================================
procedure TStack.Flush;
  var i : Integer;
begin
  with FX do
  begin
    for i := 0 to Pred(Count) do TNumber(Items[i]).Free;
    Clear;
  end;
end;

//=======================================================================
//=======================================================================
function TStack.GetCount: Integer;
begin
  Result := FX.Count;
end;

//=======================================================================
// Charge la pile depuis un fichier
// Si la pile n'est pas vide, le contenu lui est ajout
//=======================================================================
procedure TStack.LoadFromFile(const FileName: string);
  const
    XMSG  = 'Mthode TStack.LoadFromFile'#13#10'Fichier non valide';
    FLAGS = fmOpenRead or fmShareDenyWrite;
  var
    F      : TFileStream;
    Delta  : Longint; //SizeOf(Delta) = 4 octets
    i, j   : Integer;
    Header : THeaderStack;
begin
  F := TFileStream.Create(ExpandFileName(FileName),FLAGS);
  try
    F.ReadBuffer(Header,SizeOf(Header));
    if Header <> ZnZHEADERSTACK then ZnZError(XMSG);
    //----- Nombre d'lments
    F.ReadBuffer(Delta,SizeOf(Delta));
    //----- Ajuste le tableau "Items" de FX
    FX.Capacity := FX.Count + Delta;
    //----- Chargement
    with FX do
    for i := 0 to Pred(Delta) do
    begin
      //----- Cration d'un nouveau nombre
      j := Add(TNumber.Create);
      //----- Chargement du nouveau nombre
      TNumber(Items[j]).LoadFromStream(F);
    end;
  finally
    F.Free;
  end;
end;

//=======================================================================
// Dpile le nombre qui est au sommet de la pile
// Renvoie TRUE si un nombre est dpil, FALSE sinon
// Pop(nil) permet de dpiler un nombre sans le recopier
//=======================================================================
function TStack.Pop(N: TNumber): Boolean;
begin
  Result := FX.Count <> 0;
  if Result then
    with FX do
    begin
      if N <> nil then Xchg(N,First);
      TNumber(First).Free;
      Delete(0);
      Capacity := Count;
    end;
end;

//=======================================================================
// Dpile le nombre qui est  la base de la pile
// Renvoie TRUE si un nombre est dpil, FALSE sinon
// PopDown(nil) permet de dpiler un nombre sans le recopier
//=======================================================================
function TStack.PopDown(N: TNumber): Boolean;
begin
  Result := FX.Count <> 0;
  if Result then
    with FX do
    begin
      if N <> nil then Xchg(N,Last);
      TNumber(Last).Free;
      Delete(Pred(Count));
      Capacity := Count;
    end;
end;

//=======================================================================
// Empile un nombre (sur le sommet de la pile)
// Ne fait rien si N = nil
//=======================================================================
procedure TStack.Push(N: TNumber);
begin
  if N <> nil then FX.Insert(0,TNumber.Copy(N));
end;

//=======================================================================
// Sauvegarde le contenu de la pile dans un fichier
//=======================================================================
procedure TStack.SaveToFile(const FileName: string);
  const FLAGS = fmCreate or fmShareExclusive;
  var
    F : TFileStream;
    i : Longint; //Doit avoir une taille de 4 octets
begin
  F := TFileStream.Create(ExpandFileName(FileName),FLAGS);
  try
    //----- En-tte 'ZnZ Stack '
    F.WriteBuffer(ZnZHEADERSTACK,SizeOf(ZnZHEADERSTACK));
    //----- Sauvegarde FX.Count
    i := FX.Count;
    F.WriteBuffer(i,SizeOf(i));
    //----- Sauvegarde les valeurs
    with FX do
      for i := 0 to Pred(Count) do TNumber(Items[i]).SaveToStream(F);
  finally
    F.Free;
  end;
end;

//=======================================================================
// Trie le contenu de la pile
// Peut tre utilis avec les fonctions Cmpn et CmpAbs de ZnZ:
// "Stack.Sort(@Cmpn)" ou "Stack.Sort(@CmpAbs)"
//=======================================================================
procedure TStack.Sort(Compare: TStackSortCompare);
begin
  FX.Sort(Compare);
end;

//////////////////////////////////////////////////////////////////////////
end. //ZnZ_Stacks
//////////////////////////////////////////////////////////////////////////

